module renderer;

import dsfml.window.all;
import dsfml.system.all;
import dsfml.graphics.all;

import consumer;
import imagecache;
import animatedsprite;

class Renderer {
    public RenderWindow app;
    public Cache cache;
    private Consumer[] draw_consumers;
    private Consumer[] event_consumers;
    
    this(char[] title, int width, int height, int depth=32) {
	this.app = new RenderWindow(VideoMode(width, height, depth), title);
	this.app.setFramerateLimit(40);

	this.cache = new Cache("."); // TODO
    }

    /* add a consumer for event handling and drawing */
    void add_consumer(Consumer consumer) {
	this.add_event_consumer(consumer);
	this.add_draw_consumer(consumer);
    }

    /* add a consumer only for event handling */
    void add_event_consumer(Consumer consumer) {
	this.event_consumers ~= consumer;
    }

    /* add a consumer only for drawing */
    void add_draw_consumer(Consumer consumer) {
	this.draw_consumers ~= consumer;
    }

    /* use this instead of this.app.draw - it contains a hook for animated sprites */
    void draw(AnimatedSprite obj) {
	obj.update();
	this.app.draw(obj);
    }

    void draw(IDrawable obj) {
	this.app.draw(obj);
    }

    void draw(Sprite obj) {
	if(cast(AnimatedSprite)obj !is null) {
	    this.draw(cast(AnimatedSprite)obj);
	}
	else {
	    this.app.draw(obj);
	}
    }

    /* start the mainloop */
    void mainloop() {
	Event evt;

	while(this.app.isOpened()) {
	    // handle all events
	    while(this.app.getEvent(evt)) {
		if (evt.Type == Event.EventType.CLOSED) {
		    this.app.close();
		} else {
		    foreach(Consumer consumer; this.event_consumers) {
			if(consumer.handle_event(evt)) break;
		    }
		}		    
	    }
	    // draw all
	    foreach(Consumer consumer; this.draw_consumers) {
		consumer.draw();
	    }
	    // display all
	    this.app.display();
	}
    }
}
